package ru.dbotthepony.mc.otm.network

import it.unimi.dsi.fastutil.io.FastByteArrayInputStream
import net.minecraft.core.BlockPos
import net.minecraft.network.FriendlyByteBuf
import net.minecraftforge.network.NetworkDirection
import net.minecraftforge.network.NetworkEvent
import org.apache.logging.log4j.LogManager
import ru.dbotthepony.mc.otm.android.feature.ItemEntityDataPacket
import ru.dbotthepony.mc.otm.block.entity.MatteryBlockEntity
import ru.dbotthepony.mc.otm.client.minecraft
import java.util.function.Supplier

class BlockEntitySyncPacket(val position: BlockPos, val buffer: ByteArray, val validBytes: Int) : MatteryPacket {
	override fun write(buff: FriendlyByteBuf) {
		buff.writeBlockPos(position)
		buff.writeBytes(buffer, 0, validBytes)
	}

	override fun play(context: Supplier<NetworkEvent.Context>) {
		context.packetHandled = true

		context.enqueueWork {
			val level = minecraft.player?.level ?: return@enqueueWork
			val blockEntity = level.getBlockEntity(position) as? MatteryBlockEntity ?: return@enqueueWork

			try {
				blockEntity.synchronizer.read(FastByteArrayInputStream(buffer, 0, validBytes))
			} catch(err: Throwable) {
				LOGGER.error("Exception while reading synchronized BlockEntity data!\nPosition: $position\nBlock: ${level.getBlockState(position)}\nBlock entity: $blockEntity", err)
				throw err
			}
		}
	}

	companion object {
		fun read(buff: FriendlyByteBuf): BlockEntitySyncPacket {
			val position = buff.readBlockPos()
			val size = buff.readableBytes()
			val array = ByteArray(size)
			buff.readBytes(array)
			return BlockEntitySyncPacket(position, array, size)
		}

		private val LOGGER = LogManager.getLogger()
	}
}

object WorldNetworkChannel : MatteryNetworkChannel(
	version = "2",
	name = "world"
) {
	fun register() {
		add(BlockEntitySyncPacket::class.java, BlockEntitySyncPacket.Companion::read, NetworkDirection.PLAY_TO_CLIENT)
		add(ItemEntityDataPacket::class.java, ItemEntityDataPacket.Companion::read, NetworkDirection.PLAY_TO_CLIENT)
	}
}
